require 'logger'
require 'uri'
require 'net/http'
require 'json'

module Glottis
  module Handlers
    # This class manages a TCP connection with a valyx server.
    class RemoteOutputHandler < Thread
      POLL_INTERVAL = 0.1
      PROTOCOL = 'http'.freeze

      REMOTE_PATHS = {
        get_session: '/api/session',
        post_message: '/api/message'
      }.freeze

      def initialize(outgoing, host, port)
        @outgoing = outgoing
        @host = host
        @port = port

        setup_http

        super do
          @http.start
          request_session

          loop do
            send_queued
          end
        end
      end

      def cleanup
        @http.finish if @http.started?
      end

      private

      def setup_http
        @http = Net::HTTP.new(@host, @port)
        # TODO: make these configurable somewhere
        @http.open_timeout = 1
        @http.read_timeout = 1
      end

      def request_session
        Client.logger.info('making initial request for session...')

        session_req = @http.get(REMOTE_PATHS.fetch(:get_session))

        response = JSON.parse(session_req.body)

        if Integer(session_req.code) != 201
          raise "failed to retrieve session: #{response}"
        end

        @sid = response['sid']
        Client.logger.info("current session id: #{@sid}")
      end

      # given the result of a .get or .post, converts the response code,
      # and executes the (side effecting) block if the response was not
      # successful. calls the block with a message describing the error.
      def if_not_ok(http_req, ok_code = 200, &_b)
        yield http_req.body if Integer(http_req.code) != ok_code
      end

      def send_queued
        Client.logger.info("processing message with #{@outgoing.size} messages outgoing")

        message_data = {
          from: @sid,
          to: '*',
          msg: @outgoing.pop
        }.to_json

        post_req = @http.post(REMOTE_PATHS.fetch(:post_message), message_data)

        if_not_ok(post_req) do |err|
          Client.logger.warn("failed to post message: #{message_data} #{err}")
        end
      end # send_queued
    end # RemoteOutputHandler
  end # Handlers
end # Glottis
